--Update database version

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[sp_DatabaseVersion]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[sp_DatabaseVersion]
GO
CREATE PROCEDURE dbo.sp_DatabaseVersion
AS
	RETURN 95
GO

--Bug 35530

if not exists (select * from sysobjects where id=object_id('[dbo].[tblUniqueSequence]') and OBJECTPROPERTY(id, 'IsUserTable')=1)
BEGIN
	CREATE TABLE [dbo].[tblUniqueSequence] (
		[Name] [nvarchar] (255) NOT NULL ,
		[LastValue] [int] NOT NULL DEFAULT (0),
		CONSTRAINT [PK_tblUniqueSequence] PRIMARY KEY CLUSTERED 
		(
			[Name]
		)  ON [PRIMARY] 
	) ON [PRIMARY]
END
GO


SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[netUniqueSequenceNext]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[netUniqueSequenceNext]
GO

CREATE PROCEDURE dbo.netUniqueSequenceNext
@Name NVARCHAR(100),
@Steps INT,
@NextValue INT OUTPUT
AS
BEGIN
	SET NOCOUNT ON
	SET XACT_ABORT ON
	SET TRANSACTION ISOLATION LEVEL SERIALIZABLE 
	
	BEGIN TRANSACTION 
	SELECT @NextValue=LastValue + @Steps FROM tblUniqueSequence WHERE Name=@Name
	IF @@ROWCOUNT=0
	BEGIN
		INSERT INTO tblUniqueSequence(Name,LastValue) VALUES(@Name,@Steps)
		SET @NextValue = @Steps
	END
	ELSE
	BEGIN
		UPDATE tblUniqueSequence SET LastValue = @NextValue WHERE Name=@Name
	END
	COMMIT TRANSACTION
END
GO

--Bug 45038

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[netPageCopy]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[netPageCopy]
GO

CREATE PROCEDURE dbo.netPageCopy
(
	@PageID					INT,
	@DestinationPageID		INT,
	@SID					INT = 3,
	@WastebasketID			INT = NULL,
	@ForcePendingPublish	INT = 0,
	@BaseFolderID			INT = NULL,
	@CreatedPageID			INT = NULL OUTPUT
)
AS
BEGIN
	DECLARE @CopiedPageID INT
	DECLARE @Delete INT
	
	DECLARE @NewPageID INT
	DECLARE @FormID	INT
	DECLARE @NewFormID INT


	SET NOCOUNT ON
	SET XACT_ABORT ON

/*== SETUP ==*/
	
	CREATE TABLE #PageMap(fkSourcePageID INT,fkDestinationPageID INT)
	CREATE TABLE #WorkMap(fkSourceWorkID INT,fkDestinationWorkID INT)

	IF @BaseFolderID IS NULL
		SELECT @BaseFolderID=Max(ExternalFolderID)+1 FROM tblPage

	/* Handle copy from/to wastebasket */
	SET @Delete=0
	IF @WastebasketID IS NOT NULL AND (@WastebasketID=@DestinationPageID OR EXISTS(SELECT * FROM tblTree WHERE fkParentID=@WastebasketID AND fkChildID=@DestinationPageID))
			SET @Delete=1
 
	SET TRANSACTION ISOLATION LEVEL SERIALIZABLE
 
	BEGIN TRAN


/*== CREATE PAGES ==*/

		/* Copy single root page to destination*/
		INSERT INTO tblPage
			(fkParentID,		fkPageTypeID,fkCreatorID,fkChangedByID,fkFrameID,Name,LinkURL,VisibleInMenu,UseOutputCache,Deleted,AutomaticLink,FetchData,Created,Changed,StartPublish,StopPublish,ChildOrderRule,PeerOrder,fkLanguageID,PendingPublish,fkMasterLanguageBranchID,ExternalFolderID)
		SELECT
			@DestinationPageID,	fkPageTypeID,fkCreatorID,fkChangedByID,fkFrameID,Name,LinkURL,VisibleInMenu,UseOutputCache,@Delete,AutomaticLink,FetchData,Created,Changed,StartPublish,StopPublish,ChildOrderRule,PeerOrder,fkLanguageID,PendingPublish,fkMasterLanguageBranchID,@BaseFolderID
		FROM tblPage
		WHERE tblPage.pkID=@PageID

		SET @CopiedPageID = @@IDENTITY

		INSERT INTO #PageMap(fkSourcePageID,fkDestinationPageID)
		SELECT @PageID,@CopiedPageID

		/* Copy all children in tree to be direct children of copied page*/
		/* IMPORTANT! ExternalFolderID must be ZERO and fkPageLinkID must store source page */
		INSERT INTO tblPage
			(fkParentID,		fkPageLinkID,fkPageTypeID,fkCreatorID,fkChangedByID,fkFrameID,Name,LinkURL,VisibleInMenu,UseOutputCache,Deleted,AutomaticLink,FetchData,Created,Changed,StartPublish,StopPublish,ChildOrderRule,PeerOrder,fkLanguageID,PendingPublish,fkMasterLanguageBranchID,ExternalFolderID)
		SELECT
			@CopiedPageID,	tblPage.pkID,fkPageTypeID,fkCreatorID,fkChangedByID,fkFrameID,Name,LinkURL,VisibleInMenu,UseOutputCache,Deleted,AutomaticLink,FetchData,Created,Changed,StartPublish,StopPublish,ChildOrderRule,PeerOrder,fkLanguageID,PendingPublish,fkMasterLanguageBranchID,0
		FROM tblPage
		INNER JOIN tblTree ON tblTree.fkChildID=tblPage.pkID
		WHERE tblTree.fkParentID=@PageID
		AND EXISTS(SELECT * FROM tblACL INNER JOIN tblSIDGroup ON (tblSIDGroup.fkSIDParent=tblACL.fkSID OR tblACL.fkSID=1) WHERE (tblSIDGroup.fkSID=@SID OR tblACL.fkSID=1) AND tblACL.fkItemID=tblPage.pkID AND ((tblPage.PendingPublish=0 AND (Access & 1)=1) OR (tblPage.PendingPublish=1 AND (Access & 4)= 4)))

		/* Add new pages to map table */
		INSERT INTO #PageMap(fkSourcePageID,fkDestinationPageID)
		SELECT fkPageLinkID,pkID
		FROM tblPage
		WHERE tblPage.fkParentID=@CopiedPageID

		/* Set new parent and update ExternalFolderID which is later used to determine copy status*/
		UPDATE tblPage SET
			fkParentID = #PageMap.fkDestinationPageID,
			@BaseFolderID = @BaseFolderID + 1,
			ExternalFolderID = @BaseFolderID,
			fkPageLinkID = NULL /*Reset temporary variable*/
		FROM tblPage
		INNER JOIN tblPage AS OriginalPage ON OriginalPage.pkID=tblPage.fkPageLinkID
		INNER JOIN #PageMap ON #PageMap.fkSourcePageID=OriginalPage.fkParentID
		WHERE tblPage.fkParentID=@CopiedPageID

		/* Delete all children who have not been set a ExternalFolderID from above (pages that cannot be mapped to a parent caused by ACL!) */ 
		DELETE FROM #PageMap WHERE EXISTS(SELECT * FROM tblPage WHERE ExternalFolderID=0 AND tblPage.pkID=#PageMap.fkDestinationPageID)
		DELETE FROM tblPage WHERE ExternalFolderID=0 AND fkParentID=@CopiedPageID

/*==COPY DATA ==*/

		/* Copy ACL from destination page to all children (_not_ a copy of source acl)*/
		INSERT INTO tblACL(fkItemID,fkSID,Access)
		SELECT #PageMap.fkDestinationPageID,fkSID,Access
		FROM #PageMap
		LEFT JOIN tblACL ON tblACL.fkItemID=@DestinationPageID
		WHERE NOT fkSID IS NULL

		/* tblPageLanguage, language branches*/
		INSERT INTO tblPageLanguage
			(fkPageID, fkLanguageBranchID,fkCreatorID,fkChangedByID,fkPageLinkID,fkFrameID,Name,LinkURL,ExternalURL,AutomaticLink,FetchData,PendingPublish,Created,Changed,Saved,StartPublish,StopPublish,PublishedVersion,fkLanguageID,URLSegment)
		 SELECT
		   #PageMap.fkDestinationPageID,fkLanguageBranchID,fkCreatorID,fkChangedByID,fkPageLinkID,fkFrameID,Name,LinkURL,ExternalURL,AutomaticLink,FetchData,PendingPublish,Created,Changed,Saved,StartPublish,StopPublish,PublishedVersion,fkLanguageID,URLSegment 
		FROM tblPageLanguage
		INNER JOIN #PageMap ON #PageMap.fkSourcePageID=tblPageLanguage.fkPageID

		/* tblProperty, properties on pages*/
		INSERT INTO tblProperty
			(fkPageDefinitionID,fkPageID,					fkLanguageBranchID,Boolean,Number,FloatNumber,PageType,PageLink,Date,String,LongString)
		 SELECT
			fkPageDefinitionID,#PageMap.fkDestinationPageID,fkLanguageBranchID,Boolean,Number,FloatNumber,PageType,PageLink,Date,String,LongString
		FROM tblProperty
		INNER JOIN #PageMap ON #PageMap.fkSourcePageID=tblProperty.fkPageID

		/* tblPageKeyword, indexed keywords*/
		INSERT INTO tblPageKeyword
			(fkPageID,fkKeywordID,WordCount)
		SELECT
		   #PageMap.fkDestinationPageID,fkKeywordID,WordCount
		FROM tblPageKeyword
		INNER JOIN #PageMap ON #PageMap.fkSourcePageID=tblPageKeyword.fkPageID

		/* tblPageSoftlink, links on pages*/
		INSERT INTO tblPageSoftlink
		   (fkPageID,fkPageLinkID,LinkURL,LinkType,LinkProtocol)
		SELECT
			#PageMap.fkDestinationPageID,fkPageLinkID,LinkURL,LinkType,LinkProtocol
		FROM tblPageSoftlink
		INNER JOIN #PageMap ON #PageMap.fkSourcePageID=tblPageSoftlink.fkPageID

		/* tblCategoryPage, categories on published pages*/
		INSERT INTO tblCategoryPage
			(fkPageID,fkCategoryID,CategoryType,fkLanguageBranchID)
		SELECT
			#PageMap.fkDestinationPageID,fkCategoryID,CategoryType,fkLanguageBranchID
		FROM tblCategoryPage
		INNER JOIN #PageMap ON #PageMap.fkSourcePageID=tblCategoryPage.fkPageID

/*== UPDATE TREE INFORMATION ==*/

		/* Clone all copied items to the destination subtree */
		INSERT INTO tblTree(fkParentID,	fkChildID,NestingLevel,ManualInsert)
		SELECT #PageMap.fkDestinationPageID,ChildMap.fkDestinationPageID,NestingLevel,ManualInsert
		FROM tblTree
		INNER JOIN #PageMap ON #PageMap.fkSourcePageID=tblTree.fkParentID
		INNER JOIN #PageMap AS ChildMap ON ChildMap.fkSourcePageID=tblTree.fkChildID

		/* Now clone the subtree in tblTree for all parents on the destination */
		INSERT INTO tblTree
			(fkParentID,fkChildID,NestingLevel,ManualInsert)
		SELECT 
			ParentTree.fkParentID,CopiedTree.fkChildID,ParentTree.NestingLevel + CopiedTree.NestingLevel,0
		FROM (SELECT * FROM tblTree WHERE fkChildID=@DestinationPageID) AS ParentTree
		CROSS JOIN #PageMap
		INNER JOIN tblTree AS CopiedTree ON CopiedTree.fkChildID=#PageMap.fkDestinationPageID
		WHERE CopiedTree.fkParentID=@CopiedPageID

		/* Copy parent-child relations of destination to copied pages but with extra nesting level*/
		INSERT INTO tblTree
			(fkParentID,fkChildID,NestingLevel)
		SELECT
			fkParentID,@CopiedPageID,NestingLevel + 1
		FROM tblTree
		WHERE fkChildID=@DestinationPageID

		/* Finally add parent-child relation between copied page and destination */
		INSERT INTO tblTree
			(fkParentID,fkChildID,NestingLevel,ManualInsert)
		VALUES
			(@DestinationPageID,@CopiedPageID,1,0)
	
/*== COPY VERSIONS ==*/

		/* Copy published versions or if pending publish copy all versions */
		INSERT INTO tblWorkPage
			(fkPageID,fkMasterVersionID,		fkChangedByID,fkNewStatusByID,fkPageLinkID,fkFrameID,fkArchivePageID,Name,LinkURL,ExternalURL,ImageURL,VisibleInMenu,UseOutputCache,LinkType,Created,Saved,StartPublish,StopPublish,ChildOrderRule,PeerOrder,ReadyToPublish,ChangedOnPublish,HasBeenPublished,Rejected,DelayedPublish,RejectComment,fkLanguageID,fkLanguageBranchID,URLSegment)
		SELECT 
			#PageMap.fkDestinationPageID,pkID,  fkChangedByID,fkNewStatusByID,fkPageLinkID,fkFrameID,fkArchivePageID,Name,LinkURL,ExternalURL,ImageURL,VisibleInMenu,UseOutputCache,LinkType,Created,Saved,StartPublish,StopPublish,ChildOrderRule,PeerOrder,ReadyToPublish,ChangedOnPublish,HasBeenPublished,Rejected,DelayedPublish,RejectComment,fkLanguageID,fkLanguageBranchID,URLSegment
		FROM tblWorkPage
		INNER JOIN #PageMap ON #PageMap.fkSourcePageID=tblWorkPage.fkPageID
		WHERE EXISTS(SELECT * FROM tblPageLanguage WHERE tblPageLanguage.fkPageID=tblWorkPage.fkPageID AND tblPageLanguage.fkLanguageBranchID=tblWorkPage.fkLanguageBranchID AND (tblPageLanguage.PendingPublish=1 OR tblPageLanguage.PublishedVersion=tblWorkPage.pkID))

		/* Populate version mapping table */
		INSERT INTO #WorkMap(fkDestinationWorkID,fkSourceWorkID)
		SELECT pkID,fkMasterVersionID
		FROM tblWorkPage
		INNER JOIN #PageMap ON #PageMap.fkDestinationPageID=tblWorkPage.fkPageID

		/* Copy properties for copied versions */
		INSERT INTO tblWorkProperty
		   (fkPageDefinitionID,fkWorkPageID,Boolean,Number,FloatNumber,PageType,PageLink,Date,String,LongString)
		SELECT 
			fkPageDefinitionID,#WorkMap.fkDestinationWorkID,Boolean,Number,FloatNumber,PageType,PageLink,Date,String,LongString
		FROM tblWorkProperty
		INNER JOIN tblWorkPage ON tblWorkPage.pkID=tblWorkProperty.fkWorkPageID
		INNER JOIN #WorkMap ON #WorkMap.fkSourceWorkID=tblWorkPage.pkID

		/* Raw copy of all rows in tblCategoryPage*/
		INSERT INTO tblWorkCategory
			(fkWorkPageID,fkCategoryID,CategoryType)
		SELECT
			#WorkMap.fkDestinationWorkID,fkCategoryID,CategoryType
		FROM tblWorkCategory
		INNER JOIN #WorkMap ON #WorkMap.fkSourceWorkID=tblWorkCategory.fkWorkPageID

/*== UPDATE FORMS ==*/
		DECLARE cur CURSOR LOCAL FAST_FORWARD FOR 
			SELECT Number, #PageMap.fkDestinationPageID 
			FROM tblProperty
			INNER JOIN tblPageDefinition ON tblPageDefinition.pkID=tblProperty.fkPageDefinitionID
			INNER JOIN #PageMap ON #PageMap.fkSourcePageID=tblProperty.fkPageID
			WHERE tblPageDefinition.Property=9
		OPEN cur
		FETCH NEXT FROM cur INTO @FormID, @NewPageID
		WHILE (@@FETCH_STATUS = 0)
		BEGIN
			EXEC @NewFormID=editFormCopy @FormID=@FormID,@PageID=@NewPageID
			UPDATE tblProperty 
				set Number=@NewFormID
			WHERE fkPageID=@NewPageID and Number=@FormID

			/* set all new versions to point at the new form */
			UPDATE tblWorkProperty 
				set Number=@NewFormID
			FROM tblWorkProperty
			INNER JOIN tblPageDefinition ON tblPageDefinition.pkID=tblWorkProperty.fkPageDefinitionID
			INNER JOIN tblWorkPage ON tblWorkPage.pkID=tblWorkProperty.fkWorkPageID
			WHERE tblWorkPage.fkPageID=@NewPageID
			AND tblPageDefinition.Property=9

			FETCH NEXT FROM cur INTO @FormID, @NewPageID
		END
		CLOSE cur
		DEALLOCATE cur

/*== UPDATE REFERENCES ==*/

		/*Fix references in tblPage*/
		UPDATE tblPage SET 
			fkPageLinkID	= COALESCE(PageLinkMap.fkDestinationPageID,fkPageLinkID),
			fkArchivePageID	= COALESCE(ArchivePageMap.fkDestinationPageID,fkArchivePageID)
		FROM tblPage
		INNER JOIN #PageMap AS CopiedPageMap ON CopiedPageMap.fkDestinationPageID=tblPage.pkID
		LEFT JOIN #PageMap AS PageLinkMap ON PageLinkMap.fkSourcePageID=tblPage.fkPageLinkID
		LEFT JOIN #PageMap AS ArchivePageMap ON ArchivePageMap.fkSourcePageID=tblPage.fkArchivePageID

		/* Fix references in tblPageLanguage */
		UPDATE tblPageLanguage SET 
			PublishedVersion=#WorkMap.fkDestinationWorkID,
			fkPageLinkID	= COALESCE(PageLinkMap.fkDestinationPageID,fkPageLinkID)
		FROM tblPageLanguage
		INNER JOIN #PageMap ON #PageMap.fkDestinationPageID=tblPageLanguage.fkPageID
		LEFT JOIN #WorkMap ON #WorkMap.fkSourceWorkID=tblPageLanguage.PublishedVersion
		LEFT JOIN #PageMap AS PageLinkMap ON PageLinkMap.fkSourcePageID=tblPageLanguage.fkPageLinkID

		/* Fix references in tblProperty */
		UPDATE tblProperty SET 
			PageLink=#PageMap.fkDestinationPageID
		FROM tblProperty
		INNER JOIN #PageMap AS CopiedPageMap ON CopiedPageMap.fkDestinationPageID=tblProperty.fkPageID
		INNER JOIN #PageMap ON #PageMap.fkSourcePageID=tblProperty.PageLink

		/* Fix references in tblWorkPage */
		UPDATE tblWorkPage SET
			fkPageLinkID	= COALESCE(PageLinkMap.fkDestinationPageID,fkPageLinkID),
			fkArchivePageID	= COALESCE(ArchivePageMap.fkDestinationPageID,fkArchivePageID),
			fkMasterVersionID = NULL
		FROM tblWorkPage
		INNER JOIN #PageMap AS CopiedPageMap ON CopiedPageMap.fkDestinationPageID=tblWorkPage.fkPageID
		LEFT JOIN #PageMap AS PageLinkMap ON PageLinkMap.fkSourcePageID=tblWorkPage.fkPageLinkID
		LEFT JOIN #PageMap AS ArchivePageMap ON ArchivePageMap.fkSourcePageID=tblWorkPage.fkArchivePageID

		/* Fix references in tblWorkProperty */
		UPDATE tblWorkProperty SET 
			PageLink = #PageMap.fkDestinationPageID
		FROM tblWorkProperty
		INNER JOIN #WorkMap AS CopiedWorkMap ON CopiedWorkMap.fkDestinationWorkID=tblWorkProperty.fkWorkPageID
		INNER JOIN #PageMap ON #PageMap.fkSourcePageID=tblWorkProperty.PageLink

		IF @ForcePendingPublish=1
		BEGIN
			UPDATE tblPage SET 
				PendingPublish	= 1
			FROM tblPage
			INNER JOIN #PageMap AS CopiedPageMap ON CopiedPageMap.fkDestinationPageID=tblPage.pkID

			UPDATE tblPageLanguage SET 
				PendingPublish	= 1
			FROM tblPageLanguage
			INNER JOIN #PageMap AS CopiedPageMap ON CopiedPageMap.fkDestinationPageID=tblPageLanguage.fkPageID

			UPDATE tblWorkPage SET
				HasBeenPublished = 0
			FROM tblWorkPage
			INNER JOIN #PageMap AS CopiedPageMap ON CopiedPageMap.fkDestinationPageID=tblWorkPage.fkPageID
		END

		/* Fix URL segment in tblPageLanguage */
		UPDATE tblPageLanguage set URLSegment = '' where fkPageID = @CopiedPageID
		UPDATE tblWorkPage set URLSegment = '' where fkPageID = @CopiedPageID

		/* EPiServer 3 Compatibility*/
		EXEC editAddChangeLog @PageID = @DestinationPageID, @ChangeType = 3 	-- 3=Childs changed
		
	COMMIT TRAN
  
 	SET @CreatedPageID = @CopiedPageID

	/* 
		DEBUG
		SELECT *
		FROM tblPage
		INNER JOIN #PageMap ON #PageMap.fkDestinationPageID=tblPage.pkID
		SELECT *
		FROM tblTree
		INNER JOIN #PageMap ON #PageMap.fkDestinationPageID=tblTree.fkChildID
	*/

	
	DROP TABLE #PageMap
	DROP TABLE #WorkMap

	EXEC admRebuildAllLinkURL
 
	RETURN 0
END
GO

--Bug 45308

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[netKeywordSave]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[netKeywordSave]
GO
CREATE PROCEDURE dbo.netKeywordSave
(
	@PageID INT
)
AS
BEGIN
	SET NOCOUNT ON

	/*
		Handle keywords
	*/
	INSERT INTO tblKeyword (Keyword, fkLanguageBranchID)
	SELECT DISTINCT tblTempKeyword.Keyword, tblTempKeyword.fkLanguageBranchID
	FROM tblTempKeyword
	LEFT JOIN tblKeyword 
		ON tblKeyword.Keyword = tblTempKeyword.Keyword 
		AND tblKeyword.fkLanguageBranchID = tblTempKeyword.fkLanguageBranchID
	WHERE tblKeyword.Keyword IS NULL 
	AND PageID=@PageID
	
	/*
     * Only erase keywords for language branches defined in tblTempkeyword
     */
	DELETE FROM tblPageKeyword 
	FROM tblPageKeyword 
	JOIN tblKeyword ON tblKeyword.pkID=tblPageKeyword.fkKeywordID
	WHERE fkPageID=@PageID
	AND EXISTS( SELECT * FROM tblTempKeyword WHERE fkPageID=@PageID AND tblTempKeyword.fkLanguageBranchID=tblKeyword.fkLanguageBranchID)
	
	INSERT INTO tblPageKeyword(fkPageID,fkKeywordID,WordCount)
	SELECT DISTINCT @PageID, tblKeyword.pkID, Sum(tblTempKeyword.WordCount)
	FROM tblTempKeyword
	INNER JOIN tblKeyword 
		ON tblKeyword.Keyword = tblTempKeyword.Keyword 
		AND PageID=@PageID
		AND tblKeyword.fkLanguageBranchID = tblTempKeyword.fkLanguageBranchID
	GROUP BY tblKeyword.pkID
	
	/*
		Remove info from temp table
	*/
	DELETE FROM tblTempKeyword WHERE PageID=@PageID
END
GO

--Bug 45298

SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[netPageDefinitionConvertList]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[netPageDefinitionConvertList]
GO

CREATE PROCEDURE dbo.netPageDefinitionConvertList
(
	@PageDefinitionID INT
)
AS
BEGIN

	SELECT 
			fkPageDefinitionID,
			fkPageID,
			NULL AS fkWorkPageID,
			NULL AS fkUserPropertyID,
			CONVERT(INT,Boolean) AS Boolean,
			Number AS IntNumber,
			FloatNumber,
			PageType,
			PageLink,
			Date AS DateValue,
			String,
			LongString,
			CONVERT(INT,0) AS DeleteProperty
	FROM tblProperty 
	WHERE fkPageDefinitionID=@PageDefinitionID

	UNION ALL
	
	SELECT 
			fkPageDefinitionID,
			NULL AS fkPageID,
			fkWorkPageID,
			NULL AS fkUserPropertyID,
			CONVERT(INT,Boolean) AS Boolean,
			Number AS IntNumber,
			FloatNumber,
			PageType,
			PageLink,
			Date AS DateValue,
			String,
			LongString,
			CONVERT(INT,0) AS DeleteProperty
	FROM tblWorkProperty 
	WHERE fkPageDefinitionID=@PageDefinitionID

	UNION ALL
	
	SELECT 
			fkPageDefinitionID,
			NULL AS fkPageID,
			NULL AS fkWorkPageID,
			pkID AS fkUserPropertyID,
			CONVERT(INT,Boolean) AS Boolean,
			Number AS IntNumber,
			FloatNumber,
			PageType,
			PageLink,
			Date AS DateValue,
			String,
			NULL AS LongString,
			CONVERT(INT,0) AS DeleteProperty
	FROM tblUserProperty 
	WHERE fkPageDefinitionID=@PageDefinitionID
END
GO

SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[netPageDefinitionConvertSave]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[netPageDefinitionConvertSave]
GO

CREATE PROCEDURE dbo.netPageDefinitionConvertSave
(
	@PageDefinitionID INT,
	@PageID INT = NULL,
	@WorkPageID INT = NULL,
	@UserPropertyID INT = NULL,
	@Type INT,
	@Boolean BIT = NULL,
	@IntNumber INT = NULL,
	@FloatNumber FLOAT = NULL,
	@PageType INT = NULL,
	@PageReference INT = NULL,
	@DateValue DATETIME = NULL,
	@String NVARCHAR(255) = NULL,
	@LongString NTEXT = NULL,
	@DeleteProperty BIT = 0
)
AS
BEGIN
	IF NOT @UserPropertyID IS NULL
	BEGIN
		IF @DeleteProperty=1 OR (@Type=0 AND @Boolean=0)
			DELETE FROM tblUserProperty 
			WHERE fkPageDefinitionID=@PageDefinitionID AND pkID=@UserPropertyID
		ELSE IF @Type>7
			DELETE FROM tblUserProperty 
			WHERE fkPageDefinitionID=@PageDefinitionID AND pkID=@UserPropertyID
		ELSE
			UPDATE tblUserProperty
				SET
					Boolean=@Boolean,
					Number=@IntNumber,
					FloatNumber=@FloatNumber,
					PageType=@PageType,
					PageLink=@PageReference,
					Date=@DateValue,
					String=@String
			WHERE fkPageDefinitionID=@PageDefinitionID AND pkID=@UserPropertyID
	END
	ELSE IF NOT @WorkPageID IS NULL
	BEGIN		
		IF @DeleteProperty=1 OR (@Type=0 AND @Boolean=0)
			DELETE FROM tblWorkProperty 
			WHERE fkPageDefinitionID=@PageDefinitionID AND fkWorkPageID=@WorkPageID
		ELSE IF @Type>7
			DELETE FROM tblWorkProperty 
			WHERE fkPageDefinitionID=@PageDefinitionID AND fkWorkPageID=@WorkPageID
		ELSE
		BEGIN
			UPDATE tblWorkProperty
				SET
					Boolean=@Boolean,
					Number=@IntNumber,
					FloatNumber=@FloatNumber,
					PageType=@PageType,
					PageLink=@PageReference,
					Date=@DateValue,
					String=@String,
					LongString=@LongString
			WHERE fkPageDefinitionID=@PageDefinitionID AND fkWorkPageID=@WorkPageID
		END
	END
	ELSE
	BEGIN
		IF @DeleteProperty=1 OR (@Type=0 AND @Boolean=0)
			DELETE FROM tblProperty 
			WHERE fkPageDefinitionID=@PageDefinitionID AND fkPageID=@PageID
		ELSE IF @Type>7
			DELETE FROM tblProperty 
			WHERE fkPageDefinitionID=@PageDefinitionID AND fkPageID=@PageID
		ELSE
		BEGIN
			UPDATE tblProperty
				SET
					Boolean=@Boolean,
					Number=@IntNumber,
					FloatNumber=@FloatNumber,
					PageType=@PageType,
					PageLink=@PageReference,
					Date=@DateValue,
					String=@String,
					LongString=@LongString
			WHERE fkPageDefinitionID=@PageDefinitionID AND fkPageID=@PageID
		END
	END
END
GO


