--Bug 35530
BEGIN
  EXECUTE IMMEDIATE 'CREATE TABLE tblUniqueSequence (
		Name VARCHAR2 (255) NOT NULL,
		LastValue NUMBER(9) DEFAULT(0) NOT NULL,
		CONSTRAINT PK_tblUniqueSequence PRIMARY KEY
		(
			Name
		)
	)';
   EXCEPTION WHEN OTHERS THEN NULL;
END;
/
SHOW ERRORS;


CREATE OR REPLACE PROCEDURE netUniqueSequenceNext
(
	p_Name				IN VARCHAR2,
	p_Steps				IN NUMBER,
	p_NextValue			OUT NUMBER
)
AS
BEGIN
	SET TRANSACTION ISOLATION LEVEL SERIALIZABLE;
	
	BEGIN
	SELECT LastValue+p_Steps INTO p_NextValue FROM tblUniqueSequence WHERE Name=p_Name;
	EXCEPTION WHEN NO_DATA_FOUND THEN
		 INSERT INTO tblUniqueSequence(Name,LastValue) VALUES(p_Name,p_Steps);
		 SELECT p_Steps INTO p_NextValue FROM dual;
	END;
	UPDATE tblUniqueSequence SET LastValue=p_NextValue WHERE Name=p_Name;
		
	COMMIT;

END;
/
SHOW ERRORS;

--Bug 45038

/*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE PROCEDURE netPageCopy
(
	p_PageID						IN NUMBER,
	p_DestinationPageID		IN NUMBER,
	p_SID							IN NUMBER DEFAULT 3,
	p_WastebasketID			IN NUMBER DEFAULT NULL,
	p_ForcePendingPublish	IN NUMBER DEFAULT 0,
	p_BaseFolderID				IN NUMBER DEFAULT NULL,
	p_CreatedPageID			OUT NUMBER
)
AS
	v_BaseFolderID NUMBER(9):=p_BaseFolderID; -- Oracle does not support assigning IN params
	v_Delete NUMBER(1)      := 0;
	v_CopiedPageID NUMBER(9);
	v_tmp NUMBER(9);
	
-- ORACLE SPECIFIC ---
	v_rowid ROWID; 
	v_value1 NUMBER(9);
	v_value2 NUMBER(9);
	
	v_FormID NUMBER(9);
	v_NewFormID NUMBER(9);
	v_NewPageID NUMBER(9);
	
	/* Set new parent and update ExternalFolderID which is later used to determine copy status*/
	CURSOR UpdateTblPageNewParent IS
		SELECT 
			   tblPage.ROWID, 
			   TT_PageMap.fkDestinationPageID
		FROM tblPage
		INNER JOIN tblPage OriginalPage ON OriginalPage.pkID=tblPage.fkPageLinkID
		INNER JOIN TT_PageMap ON TT_PageMap.fkSourcePageID=OriginalPage.fkParentID
		WHERE tblPage.fkParentID=v_CopiedPageID
	FOR UPDATE OF tblPage.fkPageLinkID, tblPage.fkArchivePageID;

	/* Fix references in tblPage */
	CURSOR UpdateTblPageRef IS
		SELECT
			  tblPage.ROWID,
			  COALESCE(PageLinkMap.fkDestinationPageID,fkPageLinkID),
			  COALESCE(ArchivePageMap.fkDestinationPageID,fkArchivePageID)
		FROM tblPage
		INNER JOIN TT_PageMap CopiedPageMap ON CopiedPageMap.fkDestinationPageID=tblPage.pkID
		LEFT JOIN TT_PageMap PageLinkMap ON PageLinkMap.fkSourcePageID=tblPage.fkPageLinkID
		LEFT JOIN TT_PageMap ArchivePageMap ON ArchivePageMap.fkSourcePageID=tblPage.fkArchivePageID
	FOR UPDATE OF tblPage.fkPageLinkID, tblPage.fkArchivePageID;

	/* Fix references in tblPageLanguage */
	CURSOR UpdateTblPageLanguageRef IS
		SELECT
			  tblPageLanguage.ROWID,
			  TT_WorkMap.fkDestinationWorkID,
			  COALESCE(PageLinkMap.fkDestinationPageID,fkPageLinkID)
		FROM tblPageLanguage
			 INNER JOIN TT_PageMap ON TT_PageMap.fkDestinationPageID=tblPageLanguage.fkPageID
			 LEFT JOIN TT_WorkMap ON TT_WorkMap.fkSourceWorkID=tblPageLanguage.PublishedVersion
			 LEFT JOIN TT_PageMap PageLinkMap ON PageLinkMap.fkSourcePageID=tblPageLanguage.fkPageLinkID
	FOR UPDATE OF tblPageLanguage.PublishedVersion, tblPageLanguage.fkPageLinkID;

	/* Fix references in tblProperty */
	CURSOR UpdateTblPropertyRef IS
		SELECT
			tblProperty.ROWID,
			 TT_PageMap.fkDestinationPageID
		FROM tblProperty
			 INNER JOIN TT_PageMap CopiedPageMap ON CopiedPageMap.fkDestinationPageID=tblProperty.fkPageID
			 INNER JOIN TT_PageMap ON TT_PageMap.fkSourcePageID=tblProperty.PageLink
	FOR UPDATE OF tblProperty.PageLink;
	
	/* Fix references in tblWorkPage */
	CURSOR UpdateTblWorkPageRef IS
		SELECT
			  tblWorkPage.ROWID,
			  COALESCE(PageLinkMap.fkDestinationPageID,fkPageLinkID),
			  COALESCE(ArchivePageMap.fkDestinationPageID,fkArchivePageID)
		FROM tblWorkPage
		INNER JOIN TT_PageMap CopiedPageMap ON CopiedPageMap.fkDestinationPageID=tblWorkPage.fkPageID
		LEFT JOIN TT_PageMap PageLinkMap ON PageLinkMap.fkSourcePageID=tblWorkPage.fkPageLinkID
		LEFT JOIN TT_PageMap ArchivePageMap ON ArchivePageMap.fkSourcePageID=tblWorkPage.fkArchivePageID
	FOR UPDATE OF tblWorkPage.fkPageLinkID, tblWorkPage.fkArchivePageID, tblWorkPage.fkMasterVersionID;

	/* Fix references in tblWorkProperty */
	CURSOR UpdateTblWorkPropertyRef IS
		SELECT
			  tblWorkProperty.ROWID,
			  TT_PageMap.fkDestinationPageID
		FROM tblWorkProperty
		INNER JOIN TT_WorkMap CopiedWorkMap ON CopiedWorkMap.fkDestinationWorkID=tblWorkProperty.fkWorkPageID
		INNER JOIN TT_PageMap ON TT_PageMap.fkSourcePageID=tblWorkProperty.PageLink
	FOR UPDATE OF tblWorkProperty.PageLink;

	/* Force pending publish for tblPage */
	CURSOR UpdateTblPagePending IS
		SELECT 
			   tblPage.ROWID,
			   1
		FROM tblPage
		INNER JOIN TT_PageMap CopiedPageMap ON CopiedPageMap.fkDestinationPageID=tblPage.pkID
	FOR UPDATE OF tblPage.PendingPublish;

	/* Force pending publish for tblPageLanguage */
	CURSOR UpdateTblPageLanguagePending IS
		SELECT 
			   tblPageLanguage.ROWID,
			   1
		FROM tblPageLanguage
		INNER JOIN TT_PageMap CopiedPageMap ON CopiedPageMap.fkDestinationPageID=tblPageLanguage.fkPageID
	FOR UPDATE OF tblPageLanguage.PendingPublish;

	/* Force pending publish for tblWorkPage */
	CURSOR UpdateTblWorkPagePending IS
		SELECT 
			   tblWorkPage.ROWID,
			   0
		FROM tblWorkPage
		INNER JOIN TT_PageMap CopiedPageMap ON CopiedPageMap.fkDestinationPageID=tblWorkPage.fkPageID
	FOR UPDATE OF tblWorkPage.HasBeenPublished;

/*== UPDATE FORMS ==*/
	CURSOR CopyForms IS
		SELECT IntNumber, TT_PageMap.fkDestinationPageID 
		FROM tblProperty
		INNER JOIN tblPageDefinition ON tblPageDefinition.pkID=tblProperty.fkPageDefinitionID
		INNER JOIN TT_PageMap ON TT_PageMap.fkSourcePageID=tblProperty.fkPageID
		WHERE tblPageDefinition.Property=9
	FOR UPDATE OF tblProperty.IntNumber;
	----------------------
BEGIN
	-- ORACLE SPECIFIC ---
	DELETE FROM TT_PageMap;
	DELETE FROM TT_WorkMap;
	----------------------

	IF v_BaseFolderID IS NULL THEN
		SELECT Max(ExternalFolderID)+1 INTO v_BaseFolderID FROM tblPage;
	END IF;

	/* Handle copy from/to wastebasket */
	SELECT count(*) INTO v_tmp FROM tblTree WHERE fkParentID=p_WastebasketID AND fkChildID=p_DestinationPageID; 
	IF p_WastebasketID IS NOT NULL AND (p_WastebasketID=p_DestinationPageID OR v_tmp > 1) THEN
			SELECT 1 INTO v_Delete FROM DUAL;
	END IF;
			
/*== CREATE PAGES ==*/

	/* Copy single root page to destination*/
	SELECT SEQ_tblPage.nextval into v_CopiedPageID FROM DUAL; 
	INSERT INTO tblPage
		(pkID, fkParentID, fkPageTypeID,fkCreatorID,fkChangedByID,fkFrameID,Name,LinkURL,VisibleInMenu,UseOutputCache,Deleted,AutomaticLink,FetchData,Created,Changed,StartPublish,StopPublish,ChildOrderRule,PeerOrder,fkLanguageID,PendingPublish,fkMasterLanguageBranchID,ExternalFolderID)
	SELECT
		v_CopiedPageID, p_DestinationPageID, fkPageTypeID,fkCreatorID,fkChangedByID,fkFrameID,Name,LinkURL,VisibleInMenu,UseOutputCache,v_Delete,AutomaticLink,FetchData,Created,Changed,StartPublish,StopPublish,ChildOrderRule,PeerOrder,fkLanguageID,PendingPublish,fkMasterLanguageBranchID,p_BaseFolderID
	FROM tblPage
	WHERE tblPage.pkID=p_PageID;

	INSERT INTO TT_PageMap(fkSourcePageID,fkDestinationPageID)
	SELECT p_PageID,v_CopiedPageID FROM DUAL;

	/* Copy all children in tree to be direct children of copied page*/
	/* IMPORTANT! ExternalFolderID must be ZERO and fkPageLinkID must store source page */
	INSERT INTO tblPage
		(pkID, fkParentID, fkPageLinkID,fkPageTypeID,fkCreatorID,fkChangedByID,fkFrameID,Name,LinkURL,VisibleInMenu,UseOutputCache,Deleted,AutomaticLink,FetchData,Created,Changed,StartPublish,StopPublish,ChildOrderRule,PeerOrder,fkLanguageID,PendingPublish,fkMasterLanguageBranchID,ExternalFolderID)
	SELECT
		 SEQ_tblPage.nextval, v_CopiedPageID, tblPage.pkID,fkPageTypeID,fkCreatorID,fkChangedByID,fkFrameID,Name,LinkURL,VisibleInMenu,UseOutputCache,Deleted,AutomaticLink,FetchData,Created,Changed,StartPublish,StopPublish,ChildOrderRule,PeerOrder,fkLanguageID,PendingPublish,fkMasterLanguageBranchID,0
	FROM tblPage
	INNER JOIN tblTree ON tblTree.fkChildID=tblPage.pkID
	WHERE tblTree.fkParentID=p_PageID
	AND EXISTS(SELECT * FROM tblACL INNER JOIN tblSIDGroup ON (tblSIDGroup.fkSIDParent=tblACL.fkSID OR tblACL.fkSID=1) WHERE (tblSIDGroup.fkSID=p_SID OR tblACL.fkSID=1) AND tblACL.fkItemID=tblPage.pkID AND ((tblPage.PendingPublish=0 AND BITAND(AccessMask, 1)=1) OR (tblPage.PendingPublish=1 AND BITAND(AccessMask, 4)= 4)));

	/* Add new pages to map table */
	INSERT INTO TT_PageMap(fkSourcePageID,fkDestinationPageID)
	SELECT fkPageLinkID,pkID
	FROM tblPage
	WHERE tblPage.fkParentID=v_CopiedPageID;

	/* Set new parent and update ExternalFolderID which is later used to determine copy status*/
	OPEN UpdateTblPageNewParent;
	LOOP
		FETCH UpdateTblPageNewParent INTO v_rowid, v_value1;
		EXIT WHEN UpdateTblPageNewParent%NOTFOUND;
			SELECT v_BaseFolderID + 1 INTO v_BaseFolderID FROM DUAL;  
			UPDATE tblPage SET
				   fkParentID = v_value1,
				   ExternalFolderID = v_BaseFolderID,
				   fkPageLinkID = NULL /*Reset temporary variable*/
		WHERE ROWID=v_rowid;
	END LOOP;
	CLOSE UpdateTblPageNewParent;
	
	/* Delete all children who have not been set a ExternalFolderID from above (pages that cannot be mapped to a parent caused by ACL!) */ 
	DELETE FROM TT_PageMap WHERE EXISTS(SELECT * FROM tblPage WHERE ExternalFolderID=0 AND tblPage.pkID=TT_PageMap.fkDestinationPageID);
	DELETE FROM tblPage WHERE ExternalFolderID=0 AND fkParentID=v_CopiedPageID;

/*==COPY DATA ==*/

	/* Copy ACL from destination page to all children (_not_ a copy of source acl)*/
	INSERT INTO tblACL(fkItemID,fkSID,AccessMask)
	SELECT TT_PageMap.fkDestinationPageID,fkSID,AccessMask
	FROM TT_PageMap
	LEFT JOIN tblACL ON tblACL.fkItemID=p_DestinationPageID
	WHERE NOT fkSID IS NULL;

	/* tblPageLanguage, language branches*/
	INSERT INTO tblPageLanguage
		(fkPageID, fkLanguageBranchID,fkCreatorID,fkChangedByID,fkPageLinkID,fkFrameID, PageGUID, Name,LinkURL,ExternalURL,AutomaticLink,FetchData,PendingPublish,Created,Changed,Saved,StartPublish,StopPublish,PublishedVersion,fkLanguageID,URLSegment)
	 SELECT
	   TT_PageMap.fkDestinationPageID,fkLanguageBranchID,fkCreatorID,fkChangedByID,fkPageLinkID,fkFrameID, sys_guid(),Name,LinkURL,ExternalURL,AutomaticLink,FetchData,PendingPublish,Created,Changed,Saved,StartPublish,StopPublish,PublishedVersion,fkLanguageID,URLSegment 
	FROM tblPageLanguage
	INNER JOIN TT_PageMap ON TT_PageMap.fkSourcePageID=tblPageLanguage.fkPageID;

	/* tblProperty, properties on pages*/
	INSERT INTO tblProperty
		(fkPageDefinitionID,fkPageID, fkLanguageBranchID,Boolean, IntNumber,FloatNumber,PageType,PageLink,DateValue,String,LongString)
	 SELECT
		fkPageDefinitionID,TT_PageMap.fkDestinationPageID,fkLanguageBranchID,Boolean,IntNumber,FloatNumber,PageType,PageLink,DateValue,String,LongString
	FROM tblProperty
	INNER JOIN TT_PageMap ON TT_PageMap.fkSourcePageID=tblProperty.fkPageID;

	/* tblPageKeyword, indexed keywords*/
	INSERT INTO tblPageKeyword
		(fkPageID,fkKeywordID,WordCount)
	SELECT
	   TT_PageMap.fkDestinationPageID,fkKeywordID,WordCount
	FROM tblPageKeyword
	INNER JOIN TT_PageMap ON TT_PageMap.fkSourcePageID=tblPageKeyword.fkPageID;

	/* tblPageSoftLink, links on pages*/
	INSERT INTO tblPageSoftLink
	   (pkID, fkPageID,fkPageLinkID,LinkURL,LinkType,LinkProtocol)
	SELECT
		SEQ_TblPageSoftLink.NextVal, TT_PageMap.fkDestinationPageID,fkPageLinkID,LinkURL,LinkType,LinkProtocol
	FROM tblPageSoftLink
	INNER JOIN TT_PageMap ON TT_PageMap.fkSourcePageID=tblPageSoftLink.fkPageID;

	/* tblCategoryPage, categories on published pages*/
	INSERT INTO tblCategoryPage
		(fkPageID,fkCategoryID,CategoryType,fkLanguageBranchID)
	SELECT
		TT_PageMap.fkDestinationPageID,fkCategoryID,CategoryType,fkLanguageBranchID
	FROM tblCategoryPage
	INNER JOIN TT_PageMap ON TT_PageMap.fkSourcePageID=tblCategoryPage.fkPageID;
	
/*== UPDATE TREE INFORMATION ==*/

	/* Clone all copied items to the destination subtree */
	INSERT INTO tblTree(fkParentID,	fkChildID,NestingLevel,ManualInsert)
	SELECT TT_PageMap.fkDestinationPageID,ChildMap.fkDestinationPageID,NestingLevel,ManualInsert
	FROM tblTree
	INNER JOIN TT_PageMap ON TT_PageMap.fkSourcePageID=tblTree.fkParentID
	INNER JOIN TT_PageMap ChildMap ON ChildMap.fkSourcePageID=tblTree.fkChildID;

	/* Now clone the subtree in tblTree for all parents on the destination */
	INSERT INTO tblTree
		(fkParentID,fkChildID,NestingLevel,ManualInsert)
	SELECT 
		ParentTree.fkParentID,CopiedTree.fkChildID,ParentTree.NestingLevel + CopiedTree.NestingLevel,0
	FROM (SELECT * FROM tblTree WHERE fkChildID=p_DestinationPageID) ParentTree
	CROSS JOIN TT_PageMap
	INNER JOIN tblTree CopiedTree ON CopiedTree.fkChildID=TT_PageMap.fkDestinationPageID
	WHERE CopiedTree.fkParentID=v_CopiedPageID;

	/* Copy parent-child relations of destination to copied pages but with extra nesting level*/
	INSERT INTO tblTree
		(fkParentID,fkChildID,NestingLevel)
	SELECT
		fkParentID,v_CopiedPageID,NestingLevel + 1
	FROM tblTree
	WHERE fkChildID=p_DestinationPageID;

	/* Finally add parent-child relation between copied page and destination */
	INSERT INTO tblTree
		(fkParentID,fkChildID,NestingLevel,ManualInsert)
	VALUES
		(p_DestinationPageID,v_CopiedPageID,1,0);
	
/*== COPY VERSIONS ==*/

	/* Copy published versions or if pending publish copy all versions */
	INSERT INTO tblWorkPage
		(pkID, fkPageID,fkMasterVersionID, fkChangedByID,fkNewStatusByID,fkPageLinkID,fkFrameID,fkArchivePageID,Name,LinkURL,ExternalURL,ImageURL,VisibleInMenu,UseOutputCache,LinkType,Created,Saved,StartPublish,StopPublish,ChildOrderRule,PeerOrder,ReadyToPublish,ChangedOnPublish,HasBeenPublished,Rejected,DelayedPublish,RejectComment,fkLanguageID,fkLanguageBranchID,URLSegment)
	SELECT 
		SEQ_tblWorkPage.nextval, TT_PageMap.fkDestinationPageID,pkID, fkChangedByID,fkNewStatusByID,fkPageLinkID,fkFrameID,fkArchivePageID,Name,LinkURL,ExternalURL,ImageURL,VisibleInMenu,UseOutputCache,LinkType,Created,Saved,StartPublish,StopPublish,ChildOrderRule,PeerOrder,ReadyToPublish,ChangedOnPublish,HasBeenPublished,Rejected,DelayedPublish,RejectComment,fkLanguageID,fkLanguageBranchID,URLSegment
	FROM tblWorkPage
	INNER JOIN TT_PageMap ON TT_PageMap.fkSourcePageID=tblWorkPage.fkPageID
	WHERE EXISTS(SELECT * FROM tblPageLanguage WHERE tblPageLanguage.fkPageID=tblWorkPage.fkPageID AND tblPageLanguage.fkLanguageBranchID=tblWorkPage.fkLanguageBranchID AND (tblPageLanguage.PendingPublish=1 OR tblPageLanguage.PublishedVersion=tblWorkPage.pkID));

	/* Populate version mapping table */
	INSERT INTO TT_WorkMap(fkDestinationWorkID,fkSourceWorkID)
	SELECT pkID,fkMasterVersionID
	FROM tblWorkPage
	INNER JOIN TT_PageMap ON TT_PageMap.fkDestinationPageID=tblWorkPage.fkPageID;

	/* Copy properties for copied versions */
	INSERT INTO tblWorkProperty
	   (fkPageDefinitionID,fkWorkPageID,Boolean,IntNumber,FloatNumber,PageType,PageLink,DateValue,String,LongString)
	SELECT 
		fkPageDefinitionID,TT_WorkMap.fkDestinationWorkID,Boolean,IntNumber,FloatNumber,PageType,PageLink,DateValue,String,LongString
	FROM tblWorkProperty
	INNER JOIN tblWorkPage ON tblWorkPage.pkID=tblWorkProperty.fkWorkPageID
	INNER JOIN TT_WorkMap ON TT_WorkMap.fkSourceWorkID=tblWorkPage.pkID;

	/* Raw copy of all rows in tblCategoryPage*/
	INSERT INTO tblWorkCategory
		(fkWorkPageID,fkCategoryID,CategoryType)
	SELECT
		TT_WorkMap.fkDestinationWorkID,fkCategoryID,CategoryType
	FROM tblWorkCategory
	INNER JOIN TT_WorkMap ON TT_WorkMap.fkSourceWorkID=tblWorkCategory.fkWorkPageID;

/*== UPDATE FORMS ==*/
	OPEN CopyForms;
	LOOP
		FETCH CopyForms INTO v_FormID, v_NewPageID;
		EXIT WHEN CopyForms%NOTFOUND;
			v_NewFormID:=editFormCopy(p_FormID=>v_FormID,p_PageID=>v_NewPageID);
			UPDATE tblProperty 
				set IntNumber=v_NewFormID
			WHERE fkPageID=v_NewPageID and IntNumber=v_FormID;

			/* set all new versions to point at the new form */
			UPDATE tblWorkProperty 
				set IntNumber=v_NewFormID
			WHERE EXISTS (
				SELECT * FROM TBLWORKPROPERTY wp1 
				INNER JOIN TBLPAGEDEFINITION ON TBLPAGEDEFINITION.pkID=wp1.fkPageDefinitionID
				INNER JOIN TBLWORKPAGE ON TBLWORKPAGE.pkID=wp1.fkWorkPageID
				WHERE TBLWORKPROPERTY.FKWORKPAGEID=wp1.FKWORKPAGEID
				AND TBLWORKPROPERTY.fkWorkPageID=wp1.fkWorkPageID
				AND TBLPAGEDEFINITION.Property=9
				AND	TBLWORKPAGE.fkPageID=v_NewPageID
			);
	END LOOP;
	CLOSE CopyForms;

/*== UPDATE REFERENCES ==*/

	/*Fix references in tblPage*/
	OPEN UpdateTblPageRef;
	LOOP
		FETCH UpdateTblPageRef INTO v_rowid, v_value1, v_value2;
		EXIT WHEN UpdateTblPageRef%NOTFOUND;
			UPDATE tblPage SET 
				fkPageLinkID = v_value1,
				fkArchivePageID	= v_value2
			WHERE ROWID=v_rowid;
	END LOOP;
	CLOSE UpdateTblPageRef;

	/* Fix references in tblPageLanguage */
	OPEN UpdateTblPageLanguageRef;
	LOOP
		FETCH UpdateTblPagelanguageRef INTO v_rowid, v_value1, v_value2;
		EXIT WHEN UpdateTblPageLanguageRef%NOTFOUND;
			UPDATE tblPageLanguage SET
				PublishedVersion = v_value1,
				fkPageLinkID	 = v_value2
			WHERE ROWID=v_rowid;
	END LOOP;
	CLOSE UpdateTblPageLanguageRef;

	/* Fix references in tblProperty */
	OPEN UpdateTblPropertyRef;
	LOOP
		FETCH UpdateTblPropertyRef INTO v_rowid, v_value1;
		EXIT WHEN UpdateTblPropertyRef%NOTFOUND;
			UPDATE tblProperty SET 
				   PageLink = v_value1
			WHERE ROWID = v_rowid;
	END LOOP;
	CLOSE UpdateTblPropertyRef;

	/* Fix references in tblWorkPage */
	OPEN UpdateTblWorkPageRef;
	LOOP
		FETCH UpdateTblWorkPageRef INTO v_rowid, v_value1, v_value2;
		EXIT WHEN UpdateTblWorkPageRef%NOTFOUND;
			UPDATE tblWorkPage SET
				   fkPageLinkID	     = v_value1,
				   fkArchivePageID	 = v_value2,
				   fkMasterVersionID = NULL
			WHERE ROWID = v_rowid;
	END LOOP;
	CLOSE UpdateTblWorkPageRef;

	/* Fix references in tblWorkProperty */
	OPEN UpdateTblWorkPropertyRef;
	LOOP
		FETCH UpdateTblWorkPropertyRef INTO v_rowid, v_value1;
		EXIT WHEN UpdateTblWorkPropertyRef%NOTFOUND;
			UPDATE tblWorkProperty SET
				   PageLink = v_value1
			WHERE ROWID=v_rowid;
	END LOOP;
	CLOSE UpdateTblWorkPropertyRef;

	IF p_ForcePendingPublish=1 THEN
		/* Force pending publish for tblPage */
		OPEN UpdateTblPagePending;
		LOOP
			FETCH UpdateTblPagePending INTO v_rowid, v_value1;
			EXIT WHEN UpdateTblPagePending%NOTFOUND;
				 UPDATE tblPage SET
					PendingPublish = 1
			WHERE ROWID=v_rowid;
		END LOOP;
		CLOSE UpdateTblPagePending;
		
		/* Force pending publish for tblPageLanguage */
		OPEN UpdateTblPageLanguagePending;
		LOOP
			FETCH UpdateTblPageLanguagePending INTO v_rowid, v_value1;
			EXIT WHEN UpdateTblPageLanguagePending%NOTFOUND;
				 UPDATE tblPageLanguage SET
					PendingPublish = 1
			WHERE ROWID=v_rowid;
		END LOOP;
		CLOSE UpdateTblPagePending;

		/* Force pending publish for tblWorkPage */
		OPEN UpdateTblWorkPagePending;
		LOOP
			FETCH UpdateTblWorkPagePending INTO v_rowid, v_value1;
			EXIT WHEN UpdateTblWorkPagePending%NOTFOUND;
				 UPDATE tblWorkPage SET
					HasBeenPublished = 0
			WHERE ROWID=v_rowid;
		END LOOP;
		CLOSE UpdateTblPagePending;
	END IF;

	/* Fix URL segment in tblPageLanguage */
	UPDATE TBLPAGELANGUAGE SET URLSegment = '' WHERE fkPageID = v_CopiedPageID;
	UPDATE tblWorkPage set URLSegment = '' where fkPageID = v_CopiedPageID;
	
	/* EPiServer 3 Compatibility*/
	--editAddChangeLog( p_DestinationPageID, 3 ); 	-- 3=Childs changed

	COMMIT;
	
	admRebuildAllLinkURL( p_PageID=>v_CopiedPageID );
	
	select v_CopiedPageID INTO p_CreatedPageID FROM DUAL;
END;
/

SHOW ERRORS;


--Bug 45308

CREATE OR REPLACE PROCEDURE netKeywordSave
(
	p_PageID IN NUMBER
)
AS
	v_rowid ROWID;	

	CURSOR DeleteTblPageKeyword IS
		SELECT tblPageKeyword.ROWID
		FROM tblPageKeyword 
		JOIN tblKeyword ON tblKeyword.pkID=tblPageKeyword.fkKeywordID
		WHERE fkPageID=p_PageID
		AND EXISTS( SELECT * FROM tblTempKeyword WHERE fkPageID=p_PageID AND tblTempKeyword.fkLanguageBranchID=tblKeyword.fkLanguageBranchID)
	FOR UPDATE OF tblPageKeyword.fkPageID, tblPageKeyword.fkKeywordID, tblPageKeyword.Wordcount;
BEGIN
	/*
		Handle keywords
	*/
	INSERT INTO tblKeyword (Keyword, fkLanguageBranchID)
	SELECT DISTINCT tblTempKeyword.Keyword, tblTempKeyword.fkLanguageBranchID
	FROM tblTempKeyword
	LEFT JOIN tblKeyword ON tblKeyword.Keyword = tblTempKeyword.Keyword 
						AND tblKeyword.fkLanguageBranchID = tblTempkeyword.fkLanguageBranchID
	WHERE (tblKeyword.Keyword IS NULL) AND (PageID=p_PageID);

	/*
     * Only erase keywords for language branches defined in tblTempkeyword
     */
    OPEN DeleteTblPageKeyword;
	LOOP
		FETCH DeleteTblPageKeyword INTO v_rowid;
		EXIT WHEN DeleteTblPageKeyword%NOTFOUND;
			DELETE FROM tblPageKeyword 
			WHERE ROWID=v_rowid;
	END LOOP;
	CLOSE DeleteTblPageKeyword;
	
	INSERT INTO tblPageKeyword(fkPageID,fkKeywordID,WordCount)
	SELECT DISTINCT p_PageID, tblKeyword.pkID, SUM(tblTempKeyword.WordCount)
	FROM tblTempKeyword
	INNER JOIN tblKeyword 
		ON tblKeyword.Keyword = tblTempKeyword.Keyword 
		AND PageID=p_PageID
		AND tblKeyword.fkLanguageBranchID=tblTempKeyword.fkLanguageBranchID
	GROUP BY tblKeyword.pkid;
	
	/*
		Remove info from temp table
	*/
	DELETE FROM tblTempKeyword WHERE PageID=p_PageID;
END;
/
SHOW ERRORS;


--Bug 45298

CREATE OR REPLACE PROCEDURE netPageDefinitionConvertList
(
	p_PageDefinitionID	IN NUMBER,
	p_RC1				OUT MISC.REFCURSOR,
	p_RC2				OUT MISC.REFCURSOR,
	p_RC3				OUT MISC.REFCURSOR
)
AS
BEGIN

	OPEN p_RC1 FOR
	SELECT 
			fkPageDefinitionID,
			fkPageID,
			NULL AS fkWorkPageID,
			NULL AS fkUserPropertyID,
			Boolean,
			IntNumber,
			FloatNumber,
			PageType,
			PageLink,
			DateValue,
			String,
			LongString,
			0 AS DeleteProperty
	FROM tblProperty 
	WHERE fkPageDefinitionID=p_PageDefinitionID;

	OPEN p_RC2 FOR
	SELECT 
			fkPageDefinitionID,
			NULL AS fkPageID,
			fkWorkPageID,
			NULL AS fkUserPropertyID,
			Boolean,
			IntNumber,
			FloatNumber,
			PageType,
			PageLink,
			DateValue,
			String,
			LongString,
			0 AS DeleteProperty
	FROM tblWorkProperty 
	WHERE fkPageDefinitionID=p_PageDefinitionID;

	OPEN p_RC3 FOR
	SELECT 
			fkPageDefinitionID,
			NULL AS fkPageID,
			NULL AS fkWorkPageID,
			pkID AS fkUserPropertyID,
			Boolean,
			IntNumber,
			FloatNumber,
			PageType,
			PageLink,
			DateValue,
			String,
			NULL AS LongString,
			0 AS DeleteProperty
	FROM tblUserProperty 
	WHERE fkPageDefinitionID=p_PageDefinitionID;
END;
/
SHOW ERRORS;

CREATE OR REPLACE PROCEDURE netPageDefinitionConvertSave
(
	p_PageDefinitionID	IN NUMBER,
	p_PageID			IN NUMBER	DEFAULT NULL,
	p_WorkPageID		IN NUMBER	DEFAULT NULL,
	p_UserPropertyID	IN NUMBER	DEFAULT NULL,
	p_Type				IN NUMBER,
	p_Boolean			IN NUMBER	DEFAULT NULL,
	p_IntNumber			IN NUMBER	DEFAULT NULL,
	p_FloatNumber		IN NUMBER	DEFAULT NULL,
	p_PageType			IN NUMBER	DEFAULT NULL,
	p_PageReference		IN NUMBER	DEFAULT NULL,
	p_DateValue			IN DATE		DEFAULT NULL,
	p_String			IN VARCHAR2 DEFAULT NULL,
	p_LongString		IN CLOB		DEFAULT NULL,
	p_DeleteProperty	IN NUMBER	DEFAULT 0
)
AS
	v_rowid ROWID;

	CURSOR UFC1 IS 
		SELECT
			tblProperty.ROWID
		FROM tblProperty
		INNER JOIN tblPage ON tblPage.pkID = tblProperty.fkPageID AND tblPage.PublishedVersion = p_WorkPageID
		WHERE fkPageDefinitionID=p_PageDefinitionID AND tblProperty.fkPageID=p_PageID
		FOR UPDATE OF
			Boolean,
			IntNumber,
			FloatNumber,
			PageType,
			PageLink,
			DateValue,
			String,
			LongString;

	CURSOR UFC2 IS 
		SELECT
			tblWorkProperty.ROWID
		FROM tblWorkProperty
		INNER JOIN tblWorkPage ON tblWorkPage.pkID = tblWorkProperty.fkWorkPageID
		INNER JOIN tblPage ON tblPage.pkID = tblWorkPage.fkPageID AND tblPage.PublishedVersion = tblWorkPage.pkID
		WHERE tblWorkProperty.fkPageDefinitionID=p_PageDefinitionID
		FOR UPDATE OF
			Boolean,
			IntNumber,
			FloatNumber,
			PageType,
			PageLink,
			DateValue,
			String,
			LongString;
BEGIN

	IF NOT p_UserPropertyID IS NULL THEN
		IF p_DeleteProperty=1 OR (p_Type=0 AND p_Boolean=0) THEN
			DELETE FROM tblUserProperty 
			WHERE fkPageDefinitionID=p_PageDefinitionID AND pkID=p_UserPropertyID;
		ELSIF p_Type>7 THEN
			DELETE FROM tblUserProperty 
			WHERE fkPageDefinitionID=p_PageDefinitionID AND pkID=p_UserPropertyID;
		ELSE
			UPDATE tblUserProperty
				SET
					Boolean=p_Boolean,
					IntNumber=p_IntNumber,
					FloatNumber=p_FloatNumber,
					PageType=p_PageType,
					PageLink=p_PageReference,
					DateValue=p_DateValue,
					String=p_String
			WHERE fkPageDefinitionID=p_PageDefinitionID AND pkID=p_UserPropertyID;
		END IF;
	ELSIF NOT p_WorkPageID IS NULL THEN
		IF p_DeleteProperty=1 OR (p_Type=0 AND p_Boolean=0) THEN
			DELETE FROM tblWorkProperty 
			WHERE fkPageDefinitionID=p_PageDefinitionID AND fkWorkPageID=p_WorkPageID;
		ELSIF p_Type>7 THEN
			DELETE FROM tblWorkProperty 
			WHERE fkPageDefinitionID=p_PageDefinitionID AND fkWorkPageID=p_WorkPageID;
		ELSE
		
			OPEN UFC1;
			LOOP
				FETCH UFC1 INTO v_rowid;
				EXIT WHEN UFC1%NOTFOUND;
					UPDATE tblProperty
					SET 
						Boolean=p_Boolean,
						IntNumber=p_IntNumber,
						FloatNumber=p_FloatNumber,
						PageType=p_PageType,
						PageLink=p_PageReference,
						DateValue=p_DateValue,
						String=p_String,
						LongString=p_LongString					
					WHERE ROWID=v_rowid;
			END LOOP;
			CLOSE UFC1;
			
			UPDATE tblWorkProperty
				SET
					Boolean=p_Boolean,
					IntNumber=p_IntNumber,
					FloatNumber=p_FloatNumber,
					PageType=p_PageType,
					PageLink=p_PageReference,
					DateValue=p_DateValue,
					String=p_String,
					LongString=p_LongString
			WHERE fkPageDefinitionID=p_PageDefinitionID AND fkWorkPageID=p_WorkPageID;
		END IF;
	ELSE
		IF p_DeleteProperty=1 OR (p_Type=0 AND p_Boolean=0) THEN
			DELETE FROM tblProperty 
			WHERE fkPageDefinitionID=p_PageDefinitionID AND fkPageID=p_PageID;
		ELSIF p_Type>7 THEN
			DELETE FROM tblProperty 
			WHERE fkPageDefinitionID=p_PageDefinitionID AND fkPageID=p_PageID;
		ELSE
			UPDATE tblProperty
				SET
					Boolean=p_Boolean,
					IntNumber=p_IntNumber,
					FloatNumber=p_FloatNumber,
					PageType=p_PageType,
					PageLink=p_PageReference,
					DateValue=p_DateValue,
					String=p_String,
					LongString=p_LongString
			WHERE fkPageDefinitionID=p_PageDefinitionID AND fkPageID=p_PageID;
			
			OPEN UFC2;
			LOOP
				FETCH UFC2 INTO v_rowid;
				EXIT WHEN UFC2%NOTFOUND;
					UPDATE tblWorkProperty
					SET 
						Boolean=p_Boolean,
						IntNumber=p_IntNumber,
						FloatNumber=p_FloatNumber,
						PageType=p_PageType,
						PageLink=p_PageReference,
						DateValue=p_DateValue,
						String=p_String,
						LongString=p_LongString					
					WHERE ROWID=v_rowid;
			END LOOP;
			CLOSE UFC2;

		END IF;
	END IF;
END;
/
SHOW ERRORS;

CREATE OR REPLACE PROCEDURE netPageDefinitionList
(
	p_PageTypeID	IN NUMBER DEFAULT NULL,
	p_RC1			OUT MISC.REFCURSOR
)
AS
BEGIN
	OPEN p_RC1 FOR
	SELECT tblPageDefinition.pkID AS ID,
		fkPageTypeID AS PageTypeID,
		COALESCE(fkPageDefinitionTypeID,tblPageDefinition.Property) AS PageDefinitionTypeID,
		tblPageDefinition.Name,
		COALESCE(tblPageDefinitionType.Property,tblPageDefinition.Property) AS Type,
		Required,
		Advanced,
		Searchable,
		DefaultValueType,
		EditCaption,
		HelpText,
		ObjectProgID,
		LongStringSettings,
		Boolean,
		IntNumber,
		FloatNumber,
		PageType,
		PageLink,
		DateValue,
		String,
		LongString,
		NULL AS OldType,
		FieldOrder,
		LanguageSpecific
	FROM tblPageDefinition
	LEFT JOIN tblPropertyDefault ON tblPropertyDefault.fkPageDefinitionID=tblPageDefinition.pkID
	LEFT JOIN tblPageDefinitionType ON tblPageDefinitionType.pkid=tblPageDefinition.fkPageDefinitionTypeID
	WHERE (fkPageTypeID = p_PageTypeID) OR (fkPageTypeID IS NULL AND p_PageTypeID IS NULL)
	ORDER BY FieldOrder,tblPageDefinition.Name;
END;
/
SHOW ERRORS;


CREATE OR REPLACE PROCEDURE netPageDefinitionSave
(
	p_PageDefinitionID 		IN OUT NUMBER,
	p_PageTypeID			IN NUMBER,
	p_Name					IN VARCHAR2,
	p_PageDefinitionTypeID	IN NUMBER,
	p_Required				IN NUMBER DEFAULT 0,
	p_Advanced				IN NUMBER DEFAULT NULL,
	p_Searchable			IN NUMBER DEFAULT 1,
	p_DefaultValueType		IN NUMBER DEFAULT NULL,
	p_EditCaption			IN VARCHAR2 DEFAULT NULL,
	p_HelpText				IN VARCHAR2 DEFAULT NULL,
	p_ObjectProgID			IN VARCHAR2 DEFAULT NULL,
	p_LongStringSettings	IN NUMBER DEFAULT NULL,
	p_FieldOrder			IN NUMBER DEFAULT NULL,
	p_Type 					IN OUT NUMBER,
	p_OldType				IN OUT NUMBER,
	p_LanguageSpecific		IN NUMBER DEFAULT 1
)
AS

	v_RaiseError VARCHAR(1024):=NULL; -- To store RAISERROR STRING
	GENERAL_EXCEPTION EXCEPTION;
	
	v_nextval	NUMBER(9);
	
	v_FieldOrder NUMBER(9);
	v_PageTypeID NUMBER(9);
BEGIN
	v_FieldOrder := p_FieldOrder;
	v_PageTypeID := p_PageTypeID;
	
	BEGIN
		SELECT tblPageDefinitionType.Property INTO p_OldType
		FROM tblPageDefinition
		INNER JOIN tblPageDefinitionType ON tblPageDefinitionType.pkID=tblPageDefinition.fkPageDefinitionTypeID
		WHERE tblPageDefinition.pkID=p_PageDefinitionID;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			p_OldType:=NULL;
	END;
	
	SELECT Property INTO p_Type FROM tblPageDefinitionType WHERE pkID=p_PageDefinitionTypeID;
	IF p_Type IS NULL THEN
		v_RaiseError := 'Cannot find data type';
		RAISE GENERAL_EXCEPTION;
	END IF;
	IF v_PageTypeID=0 THEN
		v_PageTypeID := NULL;
	END IF;

	IF p_PageDefinitionID=0 THEN 
		IF v_FieldOrder IS NULL THEN
			SELECT Max(FieldOrder)+1 INTO v_FieldOrder FROM tblPageDefinition;
		END IF;

		-- ORACLE SPECIFIC ---
		SELECT SEQ_tblPageDefinition.nextval INTO v_nextval FROM dual;
		----------------------
			
		INSERT INTO tblPageDefinition
		(
			pkID,
			fkPageTypeID,
			fkPageDefinitionTypeID,
			Name,
			Property,
			Required,
			Advanced,
			Searchable,
			DefaultValueType,
			EditCaption,
			HelpText,
			ObjectProgID,
			LongStringSettings,
			FieldOrder,
			LanguageSpecific
		)
		VALUES
		(
			v_nextval,
			v_PageTypeID,
			p_PageDefinitionTypeID,
			p_Name,
			p_Type,
			p_Required,
			p_Advanced,
			p_Searchable,
			p_DefaultValueType,
			p_EditCaption,
			p_HelpText,
			p_ObjectProgID,
			p_LongStringSettings,
			COALESCE(v_FieldOrder,1),
			p_LanguageSpecific
		);
		p_PageDefinitionID := v_nextval;
	ELSE
		UPDATE tblPageDefinition SET
			Name 		= p_Name,
			fkPageDefinitionTypeID	= p_PageDefinitionTypeID,
			Property 	= p_Type,
			Required 	= p_Required,
			Advanced 	= p_Advanced,
			Searchable 	= p_Searchable,
			DefaultValueType = p_DefaultValueType,
			EditCaption 	= p_EditCaption,
			HelpText 	= p_HelpText,
			ObjectProgID 	= p_ObjectProgID,
			LongStringSettings = p_LongStringSettings,
			LanguageSpecific = p_LanguageSpecific
		WHERE pkID=p_PageDefinitionID;
	END IF;
	DELETE FROM tblPropertyDefault WHERE fkPageDefinitionID=p_PageDefinitionID;
	
	IF( p_LanguageSpecific = 0 ) THEN
		DELETE FROM(
				SELECT tblProperty.*
				FROM tblProperty
				INNER JOIN tblPage ON tblPage.pkID=tblProperty.fkPageID
				WHERE fkPageDefinitionID=p_PageDefinitionID AND tblProperty.fkLanguageBranchID<>tblPage.fkMasterLanguageBranchID);
	
		DELETE  FROM(
				SELECT tblWorkProperty.*
				FROM tblWorkProperty
				INNER JOIN tblWorkPage ON tblWorkProperty.fkWorkPageID=tblWorkPage.pkID
				INNER JOIN tblPage ON tblPage.pkID=tblWorkPage.fkPageID
				WHERE fkPageDefinitionID=p_PageDefinitionID AND tblWorkPage.fkLanguageBranchID<>tblPage.fkMasterLanguageBranchID);
		
		DELETE FROM(
			SELECT 
				tblCategoryPage.*
			FROM
				tblCategoryPage
			INNER JOIN
				tblPage
			ON
				tblPage.pkID = tblCategoryPage.fkPageID
			WHERE
				CategoryType = p_PageDefinitionID
			AND
				tblCategoryPage.fkLanguageBranchID <> tblPage.fkMasterLanguageBranchID);

		DELETE FROM(
			SELECT 
				tblWorkCategory.*
			FROM
				tblWorkCategory
			INNER JOIN 
				tblWorkPage
			ON
				tblWorkCategory.fkWorkPageID = tblWorkPage.pkID
			INNER JOIN
				tblPage
			ON
				tblPage.pkID = tblWorkPage.fkPageID
			WHERE
				CategoryType = p_PageDefinitionID
			AND
				tblWorkPage.fkLanguageBranchID <> tblPage.fkMasterLanguageBranchID);

	END IF;
END;
/
SHOW ERRORS;

CREATE OR REPLACE FUNCTION sp_DatabaseVersion
RETURN NUMBER
AS
BEGIN
	RETURN 95;
END;
/
SHOW ERRORS;